@@ -1,14 +1,14 @@
 # library(evd)
 
-PLIquantile = function(order,x,y,quantilehat,deltasvector,
-                       InputDistributions,type="MOY",samedelta=TRUE){
+PLIquantile = function(order,x,y,deltasvector,InputDistributions,type="MOY",samedelta=TRUE,percentage=FALSE,nboot=0,conf=0.9){
   
   # This function allows the estimation of Density Modification Based Reliability Sensitivity Indices
   # called PLI (Perturbed-Law based sensitivity Indices) for a quantile
-  # Author: Paul Lemaitre, Bertrand Iooss and Thibault Delage
+  # Author: Paul Lemaitre, Bertrand Iooss, Thibault Delage and Roman Sueur
+  #
   # Refs : P. Lemaitre, E. Sergienko, A. Arnaud, N. Bousquet, F. Gamboa and B. Iooss.
   #        Density modification based reliability sensitivity analysis, 
-  #        Journal od Statistical Computation and Simulation, 85:1200-1223, 2015.
+  #        Journal of Statistical Computation and Simulation, 85:1200-1223, 2015.
   #        hal.archives-ouvertes.fr/docs/00/73/79/78/PDF/Article_v68.pdf.
   #
   #        R. Sueur, B. Iooss and T. Delage. 
@@ -22,17 +22,21 @@ PLIquantile = function(order,x,y,quantilehat,deltasvector,
   # order is the order of the quantile to estimate.
   # x is the matrix of simulation points coordinates (one column per variable).
   # y is the vector of model outputs.
-  # quantilehat is an estimation of the quantile of the output.
   # deltasvector is a vector containing the values of delta for which the indices will be computed
   # InputDistributions is a list of list. Each list contains, as a list, the name of the distribution to be used and the parameters.
-  #  Implemented cases so far:
-  #		for a mean perturbation: Gaussian, Uniform, Triangle, Left Trucated Gaussian, Left Truncated Gumbel
-  #		for a variance perturbation: Gaussian, Uniform
+  #   Implemented cases so far:
+  #		  - for a mean perturbation: Gaussian, Uniform, Triangle, Left Trucated Gaussian, Left Truncated Gumbel
+  #		  - for a variance perturbation: Gaussian, Uniform
   # type is a character string in which the user will specify the type of perturbation wanted. 
   #	NB: the sense of "deltasvector" varies according to the type of perturbation
   #	type can take the value "MOY",in which case deltasvector is a vector of perturbated means.
   #	type can take the value "VAR",in which case deltasvector is a vector of perturbated variances, therefore needs to be positive integers.
-  # samedelta is a boolean used with the value "MOY" for type. If it is set at TRUE, the mean perturbation will be the same for all the variables. If not, the mean perturbation will be new_mean = mean+sigma*delta where mean, sigma are parameters defined in InputDistributions and delta is a value of deltasvector. See subsection 4.3.3 of the reference for an exemple of use. 
+  # samedelta is a boolean used with the value "MOY" for type. If it is set at TRUE, the mean perturbation will be the same for all the variables. 
+  #   If not, the mean perturbation will be new_mean = mean+sigma*delta where mean, sigma are parameters defined in InputDistributions and delta is a value of deltasvector. See subsection 4.3.3 of the reference for an exemple of use. 
+  # percentage defines the formula used for the PLI. If percentage=FALSE, the classical formula used in the bibliographic references is used.
+  #   If percentage=TRUE, the PLI is given in percentage of variation of the quantile (even if it is negative).
+  # nboot is the number of bootstrap replicates
+  # conf is the required bootstrap confidence interval
   # 
   #
   ###################################
@@ -50,12 +54,9 @@ PLIquantile = function(order,x,y,quantilehat,deltasvector,
   nmbredepoints=dim(x)[1]			# number of points
   nmbrededeltas=length(deltasvector)	# number of perturbations
   
-  ys = sort(y,index.return=T) # ordered output
-  xs = x[ys$ix,] # inputs ordered by increasing output
-  
   ## some storage matrices 
-  I=matrix(0,ncol=nmbredevariables,nrow=nmbrededeltas) 
-  colnames(I)=lapply(1:nmbredevariables, function(k) paste("X",k,sep="_", collapse=""))
+  I <- J <- ICIinf <- ICIsup <- JCIinf <- JCIsup <- matrix(0,ncol=nmbredevariables,nrow=nmbrededeltas) 
+  colnames(I) <- colnames(J) <- colnames(ICIinf) <- colnames(ICIsup) <- colnames(JCIinf) <- colnames(JCIsup) <- lapply(1:nmbredevariables, function(k) paste("X",k,sep="_", collapse=""))
   
   ########################################
   ## Definition of useful functions used further on 
@@ -90,11 +91,21 @@ PLIquantile = function(order,x,y,quantilehat,deltasvector,
   ########################################
   ## Principal loop of the function 
   ########################################
+  
+  quantilehat <- quantile(y,order) # quantile estimate
+  ys = sort(y,index.return=T) # ordered output
+  xs = x[ys$ix,] # inputs ordered by increasing output
+  
 for (i in 1:nmbredevariables){		# loop for each variable
   ## definition of local variables
   Loi.Entree=InputDistributions[[i]]
   lqid=rep(0,length=nmbrededeltas) 
   
+  if (nboot > 0){
+    lqidb=matrix(0,nrow=nmbrededeltas,ncol=nboot) # pour bootstrap 
+    quantilehatb=NULL
+  }
+  
   if (type=="MOY"){
     ############################
     ## definition of local variable vdd
@@ -322,7 +333,44 @@ for (i in 1:nmbredevariables){		# loop for each variable
         lqid[K] = ys$x[kid]
       } else lqid[K] = quantilehat
     }
-  }
+    
+    ###############################################################
+    ##########              BOOTSTRAP                ###############
+    ###############################################################
+    
+    if (nboot >0){
+      for (b in 1:nboot){
+        ib <- sample(1:length(y),replace=TRUE)
+        xb <- x[ib,]
+        yb <- y[ib]
+    
+        quantilehatb <- c(quantilehatb, quantile(yb,order)) # quantile estimate
+        ysb = sort(yb,index.return=T) # ordered output
+        xsb = xb[ysb$ix,] # inputs ordered by increasing output
+      
+        for (K in 1:nmbrededeltas){
+          if(vdd[K]!=0){
+            res=NULL		
+            pti=phi(vlambda[K])
+            for (j in 1:nmbredepoints){	
+              res[j]=exp(vlambda[K]*xsb[j,i]-pti)
+            }
+            sum_res = sum(res)
+            kid = 1
+            res1 = res[1]
+            res2 = res1/sum_res
+            while (res2 < order){
+              kid = kid + 1
+              res1 = res1 + res[kid]
+              res2 = res1/sum_res
+            }
+            lqidb[K,b] = ysb$x[kid]
+          } else lqidb[K,b] = quantilehatb[b]
+        }
+      } # end of bootstrap loop
+    }  
+   
+  } # endif type="MOY"
   
   ######################################################################################
   
@@ -449,20 +497,95 @@ for (i in 1:nmbredevariables){		# loop for each variable
         lqid[K] = ys$x[kid]
       } else lqid[K] = quantilehat
     }
-  }
+    
+    ###############################################################
+    ##########              BOOTSTRAP                ###############
+    ###############################################################
+    
+    if (nboot >0){
+      for (b in 1:nboot){
+        ib <- sample(1:length(y),replace=TRUE)
+        xb <- x[ib,]
+        yb <- y[ib]
+        
+        quantilehatb <- c(quantilehatb, quantile(yb,order)) # quantile estimate
+        ysb = sort(yb,index.return=T) # ordered output
+        xsb = xb[ysb$ix,] # inputs ordered by increasing output
+        
+        for (K in 1:nmbrededeltas){
+          if(deltasvector[K]!=0){
+            res=0
+            pti=phi1(c(lambda1[K],lambda2[K]))
+            for (j in 1:nmbredepoints){	
+              res[j]=exp(lambda1[K]*xsb[j,i]+lambda2[K]*xsb[j,i]^2-pti) 
+            }
+            sum_res = sum(res)
+            kid = 1
+            res1 = res[1]
+            res2 = res1/sum(res)
+            while (res2 < order){
+              kid = kid + 1
+              res1 = res1 + res[kid]
+              res2 = res1/sum_res
+            }
+            lqidb[K,b] = ysb$x[kid]
+          } else lqidb[K,b] = quantilehatb[b]
+        }
+      } # end of bootstrap loop
+    }  
+    
+  } # endif TYPE="VAR"
   
   ########################################
   ## Plugging estimator of the S_i_delta indices
   ########################################	
   
   for (j in 1:length(lqid)){
-    if(lqid[j]>quantilehat){
+    if (percentage==FALSE){
+      if(lqid[j]>quantilehat){
+        I[j,i]=lqid[j]/quantilehat-1
+        J[j,i]=lqid[j]
+      } else {
+        I[j,i]=-quantilehat/lqid[j]+1
+        J[j,i]=lqid[j]
+      }
+    } else{
       I[j,i]=lqid[j]/quantilehat-1
-    } else {
-      I[j,i]=-quantilehat/lqid[j]+1
-    }	
+      J[j,i]=lqid[j]
+    }
+    
+    if (nboot > 0){
+      qinf <- quantile(lqidb[j,],(1-conf)/2)
+      qsup <- quantile(lqidb[j,],(1+conf)/2)
+      qb <- mean(quantilehatb)
+      if (percentage==FALSE){
+        if(qinf>qb){
+          ICIinf[j,i]=qinf/qb-1
+          JCIinf[j,i]=qinf
+        } else {
+          ICIinf[j,i]=-qb/qinf+1
+          JCIinf[j,i]=qinf
+        }
+        if(qsup>quantilehat){
+          ICIsup[j,i]=qsup/qb-1
+          JCIsup[j,i]=qsup
+        } else {
+          ICIsup[j,i]=-qb/qsup+1
+          JCIsup[j,i]=qsup
+        }
+      }
+      else {
+        ICIinf[j,i]=qinf/qb-1
+        JCIinf[j,i]=qinf
+        ICIsup[j,i]=qsup/qb-1
+        JCIsup[j,i]=qsup
+      }
+    }
   }
   
-}	# End for each input.
-return(I)
+  }	# End for each input
+  
+  res <- list(PLI = I, PLICIinf = ICIinf, PLICIsup = ICIsup, quantile = J, quantileCIinf = JCIinf, quantileCIsup = JCIsup)
+  
+  return(res)
 }
